/*******************************************************************************
 * Copyright (c) 2005, 2009 Spring IDE Developers
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Spring IDE Developers - initial API and implementation
 *******************************************************************************/
package org.springframework.ide.eclipse.beans.ui.editor.hyperlink.aop;

import org.eclipse.jface.text.hyperlink.IHyperlinkDetector;
import org.springframework.ide.eclipse.beans.ui.editor.hyperlink.BeanHyperlinkCalculator;
import org.springframework.ide.eclipse.beans.ui.editor.hyperlink.ClassHyperlinkCalculator;
import org.springframework.ide.eclipse.beans.ui.editor.hyperlink.NamespaceHyperlinkDetectorSupport;
import org.springframework.ide.eclipse.beans.ui.editor.namespaces.INamespaceHyperlinkDetector;

/**
 * {@link INamespaceHyperlinkDetector} responsible for the <code>aop:*</code> namespace.
 * @author Christian Dupuis
 * @author Leo Dos Santos
 * @since 2.0
 */
public class AopHyperlinkDetector extends NamespaceHyperlinkDetectorSupport implements IHyperlinkDetector {

	@Override
	public void init() {
		ClassHyperlinkCalculator javaElement = new ClassHyperlinkCalculator();
		registerHyperlinkCalculator("implement-interface", javaElement);
		registerHyperlinkCalculator("default-impl", javaElement);

		BeanHyperlinkCalculator beanRef = new BeanHyperlinkCalculator();
		registerHyperlinkCalculator("ref", beanRef);
		registerHyperlinkCalculator("advice-ref", beanRef);
		registerHyperlinkCalculator("delegate-ref", beanRef);
		registerHyperlinkCalculator("include", "name", beanRef);

		registerHyperlinkCalculator("method", new AdviceMethodHyperlinkCalculator());
		registerHyperlinkCalculator("pointcut-ref", new PointcutReferenceHyperlinkCalculator());
	}

}
